<?php
// $Id: sf_cache_ui.bundle.inc,v 1.1.2.1 2008/03/24 21:41:23 timcn Exp $

/**
 * @file
 * Provides the interface for managing the bundle entities.
 */
 

/**
 * Menu callback; displays the bundle list page.
 */
function sf_cache_ui_bundle_list() {
  // Create a raw form structure; to be filled later.
  $form = array(
    'css' => array(
      '#tree' => TRUE,
      '#theme' => 'sf_cache_ui_bundle_list_table',
      '#title' => t('CSS bundles')
    ),
    'js' => array(
      '#tree' => TRUE,
      '#theme' => 'sf_cache_ui_bundle_list_table',
      '#title' => t('JavaScript bundles')
    ),
  );

  $bundle_pool = sf_cache_bundle_load();

  foreach ($bundle_pool as $kind => $bundles) {
    foreach ($bundles as $bid => $bundle) {
      // Add the bundle to the form.
      $form[$kind][$bid] = array(
        '#tree' => TRUE,
        '#bundle' => $bundle,
        'enabled' => array(
          '#type' => 'checkbox',
          '#default_value' => $bundle['enabled'],
          '#attributes' => array(
            'title' => t('When checked, files in this bundle will be replaced by this bundle.'),
          ),
        ),
        'compress' => array(
          '#type' => 'checkbox',
          '#default_value' => $bundle['compress'],
          '#attributes' => array(
            'title' => t('Check to compress this bundle.'),
          ),
        ),
      );
    }
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Submit callback; saves the status of the bundles.
 */
function sf_cache_ui_bundle_list_submit($form_id, $form_values) {
  $bundle_pool = sf_cache_bundle_load();

  foreach ($bundle_pool as $kind => $bundles) {
    foreach ($bundles as $bid => $bundle) {
      if (isset($form_values[$kind][$bid])) {
        sf_cache_bundle_save(array(
          'bid' => $bid,
          'enabled' => (bool)$form_values[$kind][$bid]['enabled'],
          'compress' => (bool)$form_values[$kind][$bid]['compress'],
        ), FALSE);
      }
    }
  }

  // Update all files accordingly.
  sf_cache_ui_submit_update();
}

/**
 * Themes a bundle list table.
 */
function theme_sf_cache_ui_bundle_list_table($element) {
  $element['#columns'] = array(
    'bundle_enabled' => TRUE,
    'bundle_title' => TRUE,
    'compress' => TRUE,
    'file_size' => TRUE,
    'bundle_options' => TRUE,
  );

  $element['#table_type'] = 'bundle';
  $element['#empty'] = t('No bundles available.');

  // Include the table functions and render the table.
  require_once drupal_get_path('module', 'sf_cache_ui') .'/sf_cache_ui.table.inc';
  return theme('sf_cache_ui_table', $element);
}

/**
 * Menu callback; displays a form for adding or editing a bundle.
 *
 * @param $bid
 *   (Optional) The bundle ID of the bundle that is being edited.
 */
function sf_cache_ui_bundle_edit($bid = NULL) {
  $form = array();

  if (isset($bid)) {
    $bundle = sf_cache_bundle_load($bid);

    if (!$bundle) {
      drupal_set_message(t("The specified bundle doesn't exist."), 'error');
      drupal_goto('admin/build/sf_cache/bundle');
      return;
    }
  }
  else {
    $bundle = array(
      'title' => '',
      'compress' => TRUE,
    );
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('The title of the bundle.'),
    '#size' => 40,
    '#maxlength' => 255,
    '#default_value' => $bundle['title'],
    '#required' => TRUE,
  );

  $form['kind'] = array(
    '#type' => 'radios',
    '#title' => t('Type'),
    '#description' => t('Select the type of the bundle.'),
    '#options' => array(
      'css' => t('CSS'),
      'js' => t('JavaScript')
    ),
    '#required' => TRUE,
  );

  $form['compress'] = array(
    '#type' => 'checkbox',
    '#title' => t('Compress'),
    '#description' => t('Defines whether the bundle should be compressed using the <a href="@url">selected compression method</a>.', array('@url' => url('admin/build/sf_cache/config/compress'))),
    '#default_value' => $bundle['compress'],
  );

  $form['buttons'] = array(
    'submit' => array('#type' => 'submit', '#value' => t('Save bundle')),
    '#weight' => 10,
  );

  // Fill in default values when an existing bundle is edited.
  if (isset($bundle['kind'])) {
    $form['kind']['#disabled'] = TRUE;
    $form['kind']['#default_value'] = $bundle['kind'];
    $form['kind']['#description'] .= ' <em>'. t("Note: The bundle type can't be changed after creation.") .'</em>';
    $form['bid'] = array('#type' => 'value', '#value' => $bundle['bid']);

    // Show files in this bundle.
    $files = sf_cache_ui_files_by_bundle($bundle['kind']);
    $form['files'] = $files['bundles'][$bundle['bid']];
    $form['files']['#title'] = t('Files in this bundle');
    $form['files']['#columns'] = array(
      'file-indicator' => TRUE,
      'file-name' => TRUE,
      'file-size' => TRUE,
      'file-options' => TRUE,
    );
    $form['files']['#empty'] = t('There are no files in this bundle.');
    // Include the table functions and set the correct theme function.
    require_once drupal_get_path('module', 'sf_cache_ui') .'/sf_cache_ui.table.inc';
    $form['files']['#theme'] = 'sf_cache_ui_table';
    $form['files']['#table_type'] = 'file-by-bundle-read-only';

    // Add a delete link.
    $form['buttons']['delete'] = array('#value' => l(t('Delete'), 'admin/build/sf_cache/bundle/'. $bundle['bid'] .'/delete', array(), drupal_get_destination()));
  }

  return $form;
}

/**
 * Submit callback; saves a bundle edited in a form.
 */
function sf_cache_ui_bundle_edit_submit($form_id, $form_values) {
  if (!isset($form_values['bid'])) {
    sf_cache_bundle_create(array(
      'kind' => $form_values['kind'],
      'title' => $form_values['title'],
      'compress' => $form_values['compress'],
    ));
    drupal_set_message(t('The bundle %title has been created.', array('%title' => $form_values['title'])));
  }
  else {
    $bundle = sf_cache_bundle_save($form_values);

    if ($bundle['#updated']) {
      drupal_set_message(t('The bundle %title has been updated.', array('%title' => $form_values['title'])));
    }
    else {
      drupal_set_message(t('There were no changes to the bundle %title.', array('%title' => $form_values['title'])));
    }
  }

  return 'admin/build/sf_cache/bundle';
}

/**
 * Menu callback; shows a confirmation page before deleting a bundle.
 */
function sf_cache_ui_bundle_delete($bid) {
  $bundle = sf_cache_bundle_load($bid);

  if ($bundle) {
    $form = array(
      'bid' => array('#type' => 'value', '#value' => $bid),
    );

    return confirm_form($form,
       t('Are you sure you want to delete the bundle %title?', array('%title' => $bundle['title'])),
       isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/sf_cache/bundle',
       t('All files in this bundle will become standalone. This action cannot be undone.'),
       t('Delete'), t('Cancel'));
  }
  else {
    drupal_not_found();
    exit;
  }
}

/**
 * Submit callback; handles the deletion of a bundle.
 */
function sf_cache_ui_bundle_delete_submit($form_id, $form_values) {
  $bundle = sf_cache_bundle_load($form_values['bid']);
  sf_cache_bundle_delete($form_values['bid']);
  drupal_set_message(t('The bundle %title has been deleted.', array('%title' => $bundle['title'])));

  // Update all files accordingly.
  sf_cache_ui_submit_update();

  return 'admin/build/sf_cache/bundle';
}
